from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary

from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from collective.taghelper.interfaces import ITagHelperSettingsSchema


def tagging_vocabulary_factory(context):
    items = []
    items.append(('ttn', u'TagThe.Net'))
    try:
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ITagHelperSettingsSchema)
        if settings.silcc_url:
            items.append(('silcc', u'SiLLC'))
        if settings.yahoo_api_key:
            items.append(('yahoo', u'Yahoo'))
        if settings.alchemy_api_key:
            items.append(('alchemy', u'AlchemyAPI'))
        if settings.calais_api_key:
            items.append(('calais', u'Open Calais'))
        if settings.zemanta_api_key:
            items.append(('zemanta', u'Zemanta'))
        if settings.openamplify_api_key:
            items.append(('openamplify', u'OpenAmplify'))
        if settings.evri_api_key:
            items.append(('evri', u'Evri'))

    except (KeyError, AttributeError):
        return SimpleVocabulary.fromItems(items)
    return SimpleVocabulary.fromItems(items)



from plone.app.registry.browser import controlpanel

from collective.taghelper import taghelperMessageFactory as _
from collective.taghelper.interfaces import ITagHelperSettingsSchema



class TagHelperSettings(controlpanel.RegistryEditForm):
    #form_fields = form.FormFields(ITagHelperSettingsSchema)
    schema = ITagHelperSettingsSchema
    label = _(u'Tag Helper Settings')
    description = _(u'API Keys for the webservices used to extract terms')

    def updateFields(self):
        super(TagHelperSettings, self).updateFields()


    def updateWidgets(self):
        super(TagHelperSettings, self).updateWidgets()


class TagHelperSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = TagHelperSettings






from zope.interface import implements, Interface

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName

from collective.taghelper import taghelperMessageFactory as _


class IFileHtmlPreviewView(Interface):
    """
    FileHtmlPreview view interface
    """



class FileHtmlPreviewView(BrowserView):
    """
    FileHtmlPreview browser view
    """
    implements(IFileHtmlPreviewView)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_html(self):
        portal_transforms = getToolByName(self.context, 'portal_transforms')
        try:
            html = portal_transforms.convertTo('text/html',
                self.context.data,
                mimetype=self.context.get_content_type).getData()
        except AttributeError:
            html = '<h1> %s </h1> <p> %s </p>' %(self.context.Title(),
                    self.context.Description())
        return html



from zope import interface, schema

from collective.taghelper import taghelperMessageFactory as _

class ITagHelperSettingsSchema(interface.Interface):
    # -*- extra stuff goes here -*-

    calais_api_key = schema.TextLine(
        title=u'Open Calais API Key',
        description=u'API Key can be obtained at http://www.opencalais.com/APIkey',
        required=False,
        readonly=False,
        default=None,
        )

    calais_relevance = schema.Float(
        title=u'Open Calais relevance',
        description=u'1 and above',
        required=True,
        readonly=False,
        default=0.5,
        )


    yahoo_api_key = schema.TextLine(
        title=u'Yahoo API Key',
        description=u'API Key can be obtained at http://developer.apps.yahoo.com/dashboard/createKey.html',
        required=False,
        readonly=False,
        default=None,
        )

    silcc_api_key = schema.TextLine(
        title=u'SiLCC API Key',
        description=u'API Key can be obtained at http://opensilcc.com/',
        required=False,
        readonly=False,
        default=None,
        )


    silcc_url = schema.TextLine(
        title=u'SiLCC server URL',
        description=u'Enter the url of your silcc server',
        required=False,
        readonly=False,
        default=u'http://opensilcc.com/api/tag',
        )

    alchemy_api_key = schema.TextLine(
        title=u'AlchemyAPI Key',
        description=u'API Key can be obtained at http://www.alchemyapi.com/api/register.html',
        required=False,
        readonly=False,
        default=None,
        )

    alchemy_relevance = schema.Float(
        title=u'AlchemyAPI relevance',
        description=u'between 0 and 1',
        required=True,
        readonly=False,
        default=0.2,
        )


    zemanta_api_key = schema.TextLine(
        title=u'Zemanta API Key',
        description=u'API Key can be obtained at http://developer.zemanta.com/apps/register/',
        required=False,
        readonly=False,
        default=None,
        )

    zemanta_relevance = schema.Float(
        title=u'Zemanta relevance',
        description=u'between 0 and 1',
        required=True,
        readonly=False,
        default=0.05,
        )


    openamplify_api_key = schema.TextLine(
        title=u'OpenAmplify API Key',
        description=u'API Key can be obtained at http://www.openamplify.com/user',
        required=False,
        readonly=False,
        default=None,
        )

    evri_api_key = schema.TextLine(
        title=u'Evri API Key',
        description=u'API Key can be obtained at http://www.evri.com/developer/api-registration',
        required=False,
        readonly=False,
        default=None,
        )

    evri_relevance = schema.Float(
        title=u'Evri relevance',
        description=u'>>1',
        required=True,
        readonly=False,
        default=1.0,
        )

    use_remote_url = schema.Bool(
        title=u'Use remote url',
        description=u'''If the content item has a remote url
                        extract the tags from there rather than
                        from the local content (Yahoo, AlchemyAPI and tagthe.net only)''',
        required=False,
        readonly=False,
        default=True,
        )

    webservices = schema.List(
        title = _(u'Use webservices'),
        required = False,
        default = [],
        description = _(u"List the webservices you want to use for tagging"),
        value_type = schema.Choice(title=_(u"webservices"),
                    source="collective.taghelper.webservices"))



import urllib, urllib2
try:
    import simplejson as json
except ImportError:
    import json

class Zemanta(object):

    api_key = None
    url = 'http://api.zemanta.com/services/rest/0.0/'

    def __init__(self, api_key, relevance):
        self.api_key = api_key
        self.relevance = relevance

    def analyze(self, text, title=''):
        args = {'method': "zemanta.suggest",
        'api_key': self.api_key,
        'text': text.lstrip(title),
        'return_images': 0,
        'return_keywords': 1,
        'text_title': title,
        'images_limit': 0,
        'articles_limit': 0,
        'format': 'json'}
        args_enc = urllib.urlencode(args)
        raw_output = urllib.urlopen(self.url, args_enc).read()
        output = json.loads(raw_output)
        if output['status'].lower()=='ok':
            return [kw['name'] for kw in output['keywords'] if kw['confidence'] > self.relevance]
        return []



from zope.interface import implements, Interface
from zope.component import getUtility

from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from plone.registry.interfaces import IRegistry
from collective.taghelper.interfaces import ITagHelperSettingsSchema
from collective.taghelper import taghelperMessageFactory as _
from collective.taghelper.utilities import get_yql_subjects
from collective.taghelper.utilities import get_yql_subjects_remote
from collective.taghelper.utilities import get_calais_subjects
from collective.taghelper.utilities import get_silcc_subjects
from collective.taghelper.utilities import get_ttn_subjects
from collective.taghelper.utilities import get_ttn_subjects_remote
from collective.taghelper.utilities import get_alchemy_subjects
from collective.taghelper.utilities import get_alchemy_subjects_remote
from collective.taghelper.utilities import get_zemanta_subjects
from collective.taghelper.utilities import get_amplify_subjects
from collective.taghelper.utilities import get_amplify_subjects_remote
from collective.taghelper.utilities import get_evri_subjects

from collective.taghelper.interfaces import ITagHelperSettingsSchema

class IExtractedTermsView(Interface):
    """
    ExtractedTerms view interface
    """

class ExtractedTermsView(BrowserView):
    """
    ExtractedTerms browser view
    """
    implements(IExtractedTermsView)

    template = ViewPageTemplateFile('extractedtermsview.pt')
    use_remote_url = False
    url = ''

    js_template = '''
        $.get('%(url)s',
                function(data) {
                  $('#%(id)s').html(data);
            });
            '''
    def __init__(self, context, request):
        self.context = context
        self.request = request


    def services(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ITagHelperSettingsSchema)
        active_services = settings.webservices
        slist=[]
        for service in active_services:
            sname = service.replace(' ','').replace('.', '').lower()
            slist.append({'sid': sname, 'name': service})
        return slist

    def get_js(self, service):
        vars = {}
        url = '/@@tagsnippets.html?service='
        vars['url'] = self.context.absolute_url() + url + service
        vars['id'] = 'results-' + service
        return self.js_template % vars


    def __call__(self):
        form = self.request.form
        if form.has_key('form.button.save'):
            keywords = list(self.context.Subject())
            keywords = keywords + form.get('subject', [])
            keywords=list(set(keywords))
            self.context.setSubject(keywords)
            self.request.response.redirect(self.context.absolute_url() + '/edit')
            return ''
        elif form.has_key('form.button.cancel'):
            self.request.response.redirect(self.context.absolute_url() + '/view')
            return ''
        return self.template()


class IETSnippetView(Interface):
    """
    ExtractedTerms Result Snippets view interface
    """

class ETSnippetView(BrowserView):
    """
    ExtractedTerms Result Snippets browser view
    """
    implements(IETSnippetView)

    template = ViewPageTemplateFile('etsnippetview.pt')
    use_remote_url = False
    url = ''

    def __init__(self, context, request):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ITagHelperSettingsSchema)
        self.use_remote_url = settings.use_remote_url
        self.context = context
        self.request = request
        if self.context.portal_type == 'File':
            self.url = self.request.URL1 +'/filehtmlpreview_view'
        if hasattr(self.context, 'getRemoteUrl'):
            if self.use_remote_url and self.context.getRemoteUrl():
                self.url = self.context.getRemoteUrl()
            else:
                self.url = self.request.URL1 +'?ajax_load=1'
                self.use_remote_url = False
        else:
            self.use_remote_url = False
        if not self.url:
            self.url = self.request.URL1 +'?ajax_load=1'
        self.text = self._get_text()

    def _get_text(self):
        if hasattr(self.context, 'SearchableText'):
            text = self.context.SearchableText().strip().lstrip(self.context.id).encode('utf-8', 'ignore')
            text = text.lstrip().lstrip(self.context.Title().encode('utf-8', 'ignore')).encode('utf-8', 'ignore')
            text = self.context.Title().encode('utf-8', 'ignore') + u'\n\n' + text
            return text
        else:
            return ''


    @property
    def service_id(self):
        return self.request.form.get('service', '')

    def terms(self):
        sid = self.service_id
        if sid=='tagthenet':
            if self.use_remote_url:
                tags = get_ttn_subjects_remote(self.url)
            else:
                tags = get_ttn_subjects(self.text)
        elif sid=='alchemyapi':
            if self.use_remote_url:
                tags = get_alchemy_subjects_remote(self.url)
            else:
                tags = get_alchemy_subjects(self.text)
        elif sid=='yahoo':
            if self.use_remote_url:
                tags = get_yql_subjects_remote(self.url)
            else:
                 tags = get_yql_subjects(self.text)
        elif sid=='openamplify':
            if self.use_remote_url:
                tags = get_amplify_subjects_remote(self.url)
            else:
                 tags = get_amplify_subjects(self.text)
        elif sid=='evri':
            if self.use_remote_url:
                tags = get_evri_subjects(self.url)
            else:
                 tags = get_evri_subjects(self.url, self.text)
        elif sid=='opencalais':
            tags = get_calais_subjects(self.text, self.context.UID())
        elif sid=='sillc':
            text = self.context.Title() + '. ' + self.context.Description()
            tags = get_silcc_subjects(text)
        elif sid =='zemanta':
            tags = get_zemanta_subjects(self.text, self.context.Title())

        else:
            return []
        keywords = list(self.context.Subject())
        for kw in keywords:
            if kw in tags:
                tags.remove(kw)
        return tags


    def tos(self):
        sid = self.service_id
        if sid=='tagthenet':
            t = """<a href="http://www.tagthe.net/faq"> Terms of use </a>
            """
        elif sid=='alchemyapi':
            t = """<a href="http://www.alchemyapi.com/company/terms.html" >Terms of Use
            <img src="http://www.alchemyapi.com/images/alchemyAPI.jpg" alt="AlchemyAPI" />
            </a>
            """
        elif sid=='yahoo':
            t = """ <a href="http://info.yahoo.com/legal/us/yahoo/yql/yql-4307.html">
            Yahoo! Query Language Terms Of Use</a>"""
        elif sid=='opencalais':
            t =""" <a href = "http://www.opencalais.com/terms"> Calais Web Service Terms of Service
            <img src="http://www.opencalais.com/files/wpro_shared/images/Calais%20icon_16x16.jpg"
                alt="Open Calais" title="Calais powered by Thomson Reuters" />
            </a>"""
        elif sid=='sillc':
            t = ''
        elif sid =='zemanta':
            t= """<a href="http://developer.zemanta.com/API_terms_of_use/">
            Terms of Use
            <img src="http://www.zemanta.com/smedia/img/layout/logo.png" alt="Zemanta"/>
            </a>
             """
        elif sid =='openamplify':
            t= """<a href="http://openamplify.com/terms">
            OpenAmplify Terms of Service </a>
            <a href="http://www.openAmplify.com" >
            <img src="http://openamplify.com/sites/default/files/logo.png" alt="openAmplify" />
            </a>
             """
        elif sid =='evri':
            t= """<a href=https://www.webservius.com/cons-help/legal/consumer-agreement.aspx?cobrand=Evri" >
            Consumer Agreement </a>
            """
        else:
            t = ''
        return t

    def __call__(self):
        self.request.response.setHeader('X-Theme-Disabled', 'True')
        return self.template()





"""
python-calais v.1.4 -- Python interface to the OpenCalais API
Author: Jordan Dimov (jdimov@mlke.net)
Last-Update: 01/12/2009
"""

import httplib, urllib, urllib2, re
try:
    import simplejson as json
except ImportError:
    import json

from StringIO import StringIO

PARAMS_XML = """
<c:params xmlns:c="http://s.opencalais.com/1/pred/"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
    <c:processingDirectives %s> </c:processingDirectives>
    <c:userDirectives %s> </c:userDirectives>
    <c:externalMetadata %s> </c:externalMetadata>
</c:params>
"""

STRIP_RE = re.compile('<script.*?</script>|<noscript.*?</noscript>|<style.*?</style>', re.IGNORECASE)

__version__ = "1.5"

class AppURLopener(urllib.FancyURLopener):
    version = "Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.5) Gecko/2008121623 Ubuntu/8.10 (intrepid)Firefox/3.0.5" # Lie shamelessly to Wikipedia.
urllib._urlopener = AppURLopener()

class Calais():
    """
    Python class that knows how to talk to the OpenCalais API.  Use the analyze() and analyze_url() methods, which return CalaisResponse objects.
    """
    api_key = None
    processing_directives = {"contentType":"TEXT/RAW",
                    "outputFormat":"application/json",
                    "reltagBaseURL":None,
                    "calculateRelevanceScore":"true",
                    "enableMetadataType":'SocialTags',
                    "discardMetadata":None,
                    "omitOutputtingOriginalText":"true"}
    user_directives = {"allowDistribution":"false",
                    "allowSearch":"false",
                    "externalID":None}
    external_metadata = {}
    url = 'http://api.opencalais.com/enlighten/rest/'

    def __init__(self, api_key, submitter="python-calais client v.%s" % __version__):
        self.api_key = api_key
        self.user_directives["submitter"]=submitter

    def _get_params_XML(self):
        return PARAMS_XML % (" ".join('c:%s="%s"' % (k,v) for (k,v) in self.processing_directives.items() if v), " ".join('c:%s="%s"' % (k,v) for (k,v) in self.user_directives.items() if v), " ".join('c:%s="%s"' % (k,v) for (k,v) in self.external_metadata.items() if v))

    def rest_POST(self, content):
        params = urllib.urlencode({'licenseID':self.api_key,
                'content':content, 'paramsXML':self._get_params_XML()})
        #headers = {"Content-type":"application/x-www-form-urlencoded"}
        #conn = httplib.HTTPConnection("api.opencalais.com:80")
        #conn.request("POST", "/enlighten/rest/", params, headers)
        #response = conn.getresponse()
        #data = response.read()
        #conn.close()
        response = urllib2.urlopen(self.url, data=params)
        data = response.read()
        return (data)

    def get_random_id(self):
        """
        Creates a random 10-character ID for your submission.
        """
        import string
        from random import choice
        chars = string.letters + string.digits
        np = ""
        for i in range(10):
            np = np + choice(chars)
        return np

    def get_content_id(self, text):
        """
        Creates a SHA1 hash of the text of your submission.
        """
        import hashlib
        h = hashlib.sha1()
        h.update(text)
        return h.hexdigest()

    def preprocess_html(self, html):
        html = html.replace('\n', '')
        html = STRIP_RE.sub('', html)
        return html

    def analyze(self, content, content_type="TEXT/RAW", external_id=None):
        if not (content and  len(content.strip())):
            return None
        self.processing_directives["contentType"]=content_type
        if external_id:
            self.user_directives["externalID"] = external_id
        return CalaisResponse(self.rest_POST(content))

    def analyze_url(self, url):
        f = urllib.urlopen(url)
        html = self.preprocess_html(f.read())
        return self.analyze(html, content_type="TEXT/HTML", external_id=url)

    def analyze_file(self, fn):
        import mimetypes
        try:
            filetype = mimetypes.guess_type(fn)[0]
        except:
            raise ValueError("Can not determine file type for '%s'" % fn)
        if filetype == "text/plain":
            content_type="TEXT/RAW"
            f = open(fn)
            content = f.read()
            f.close()
        elif filetype == "text/html":
            content_type = "TEXT/HTML"
            f = open(fn)
            content = self.preprocess_html(f.read())
            f.close()
        else:
            raise ValueError("Only plaintext and HTML files are currently supported.  ")
        return self.analyze(content, content_type=content_type, external_id=fn)

class CalaisResponse():
    """
    Encapsulates a parsed Calais response and provides easy pythonic access to the data.
    """
    raw_response = None
    simplified_response = None

    def __init__(self, raw_result):
        try:
            self.raw_response = json.load(StringIO(raw_result))
        except:
            raise ValueError(raw_result)
        self.simplified_response = self._simplify_json(self.raw_response)
        self.__dict__['doc'] = self.raw_response['doc']
        for k,v in self.simplified_response.items():
            self.__dict__[k] = v

    def _simplify_json(self, json):
        result = {}
        # First, resolve references
        for element in json.values():
            for k,v in element.items():
                if isinstance(v, unicode) and v.startswith("http://") and json.has_key(v):
                    element[k] = json[v]
        for k, v in json.items():
            if v.has_key("_typeGroup"):
                group = v["_typeGroup"]
                if not result.has_key(group):
                    result[group]=[]
                del v["_typeGroup"]
                v["__reference"] = k
                result[group].append(v)
        return result

    def print_summary(self):
        if not hasattr(self, "doc"):
            return None
        info = self.doc['info']
        print "Calais Request ID: %s" % info['calaisRequestID']
        if info.has_key('externalID'):
            print "External ID: %s" % info['externalID']
        if info.has_key('docTitle'):
            print "Title: %s " % info['docTitle']
        print "Language: %s" % self.doc['meta']['language']
        print "Extractions: "
        for k,v in self.simplified_response.items():
            print "\t%d %s" % (len(v), k)

    def print_entities(self):
        if not hasattr(self, "entities"):
            return None
        for item in self.entities:
            print "%s: %s (%.2f)" % (item['_type'], item['name'], item['relevance'])

    def print_topics(self):
        if not hasattr(self, "topics"):
            return None
        for topic in self.topics:
            print topic['categoryName']

    def print_relations(self):
        if not hasattr(self, "relations"):
            return None
        for relation in self.relations:
            print relation['_type']
            for k,v in relation.items():
                if not k.startswith("_"):
                    if isinstance(v, unicode):
                        print "\t%s:%s" % (k,v)
                    elif isinstance(v, dict) and v.has_key('name'):
                        print "\t%s:%s" % (k, v['name'])



import logging
from Products.CMFCore.utils import getToolByName
from collective.taghelper.config import PROJECTNAME

PROFILE_ID = 'profile-%s:default' % PROJECTNAME

def upgrade_registry(context, logger=None):
    """Re-import the portal configuration registry settings.
    """
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('collective.taghelper')
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')
    logger.info("imported registry settings")
    return



import urllib, urllib2
try:
    import simplejson as json
except ImportError:
    import json

class Silcc(object):

    api_key = None
    url = 'http://opensilcc.com/api/tag'

    def __init__(self, api_key, url="http://opensilcc.com/api/tag"):
        self.api_key = api_key
        self.url = url

    def rest_POST(self, content):
        params = urllib.urlencode({'key': self.api_key, 'text': content})
        response = urllib2.urlopen(self.url, data=params)
        data = response.read()
        return (data)

    def execute(self, content):
        if not (content and  len(content.strip())):
            return None
        result = self.rest_POST(content)
        results = json.loads(result)
        return results



import urllib, urllib2
from elementtree.ElementTree import XML


class Evri(object):
    url = 'http://api.evri.com/v1/media/entities'

    def __init__(self, api_key, relevance = 1.0):
        self.api_key = api_key
        self.relevance = relevance

    def analyze(self, url, text=None):
        if text:
            args = {'uri': url,
            'appId': self.api_key,
            'text': text}
        else:
            args = {'uri': url,
            'appId': self.api_key}
        args_enc = urllib.urlencode(args)
        output = urllib.urlopen(self.url, args_enc).read()
        dom = XML(output)
        aliases = []
        for alias in dom.findall('.//alias'):
            aliases.append(alias.text)
        results = []
        for entity in dom.findall('.//entity'):
            name = entity.find('name')
            try:
                if float(entity.attrib['score']) < self.relevance:
                    continue
            except KeyError:
                pass
            if name != None:
                if name.text in aliases:
                    continue
            cname = entity.find('canonicalName')
            if cname != None:
                name = cname
            if name == None:
                continue
            else:
                if name.text in results:
                    continue
            results.append(name.text)
        return results



import urllib, urllib2
try:
    import simplejson as json
except ImportError:
    import json

class Amplify(object):

    url = "http://portaltnx20.openamplify.com/AmplifyWeb_v20/AmplifyThis"

    def __init__(self, api_key):
        self.api_key = api_key


    def amplify(self, text, analysis='topics'):
        """
        Send a request to the openamplify server
        Arguments:
          analysis - 'all','topics','actions','demographics','styles'
          output_format - 'xml','json','dart'
        """

        post_parameters = {
            'inputtext': text,
            'apiKey': self.api_key,
            'outputformat':'json',
            'analysis':analysis,
        }
        args_enc = urllib.urlencode(post_parameters)
        try:
            response = urllib2.urlopen(self.url, args_enc)
        except urllib2.HTTPError, e:
            # Check for for Forbidden HTTP status, which means invalid API key.
            if e.code == 403:
                return ['Invalid API key.']
            return ['error %i' % e.code]

        output = json.loads(response.read())
        results =[]
        try:
             results = [kw['Name'] for kw in output['ns1:TopicsResponse']['TopicsReturn']['Topics']['Locations']]
        except KeyError:
            pass
        try:
            results += [kw['Topic']['Name'] for kw in output['ns1:TopicsResponse']['TopicsReturn']['Topics']['TopTopics']]
        except KeyError:
            pass
        return results



# generate Subjects

import yql
from calais import Calais
from silcc import Silcc
from tagthenet import TagTheNet
from AlchemyAPI import AlchemyAPI
from zemanta import Zemanta
from amplify import Amplify
from evri import Evri

from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from collective.taghelper.interfaces import ITagHelperSettingsSchema

from elementtree.ElementTree import XML




PREFERRED_ENTITIES = ['City', 'Continent', 'Country', 'MedicalCondition',
    'MedicalTreatment', 'NaturalFeature', 'Organization', 'ProvinceOrState',
    'Region', 'IndustryTerm']
PREFERRED_FACTS = ['EnvironmentalIssue', 'ManMadeDisaster', 'NaturalDisaster']

def get_evri_subjects(url, text=None):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.evri_api_key
    relevance = settings.evri_relevance
    results = []
    if api_key:
        evriObj = Evri(api_key, relevance)
        results = evriObj.analyze(url, text)
    return results


def get_amplify_subjects(text):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.openamplify_api_key
    results = []
    if api_key:
        amplifyObj = Amplify(api_key)
        results = amplifyObj.amplify(text)
    return results

def get_amplify_subjects_remote(url):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.amplify_api_key
    results = []
    a_url = 'sourceURL=' + url
    if api_key:
        amplifyObj = Amplify(api_key)
        results = amplifyObj.amplify(a_url)
    return results


def get_zemanta_subjects(text, title=''):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.zemanta_api_key
    relevance = settings.zemanta_relevance
    results = []
    if api_key:
        zemantaObj = Zemanta(api_key, relevance)
        results = zemantaObj.analyze(text, title)
    return results


def _list_alchemy_results(xml,relevance):
    dom = XML(xml)
    results = []
    if dom.find('status').text == 'OK':
        for concept in dom.findall('.//concept'):
            if float(concept.find('relevance').text) > relevance:
                results.append(concept.find('text').text)
        for kw in dom.findall('.//keyword'):
            if float(kw.find('relevance').text) > relevance:
                results.append(kw.find('text').text)
    return results


def get_alchemy_subjects(text):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.alchemy_api_key
    relevance = settings.alchemy_relevance
    results = []
    if api_key:
        alchemyObj = AlchemyAPI()
        alchemyObj.setAPIKey(api_key)
        try:
            result = alchemyObj.TextGetRankedConcepts(text)
            results += _list_alchemy_results(result, relevance)
            #result = alchemyObj.TextGetRankedKeywords(text)
            #results += _list_alchemy_results(result)
            #results = list(set(results))
            return results
        except:
            return results
    else:
        return results

def get_alchemy_subjects_remote(url):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.alchemy_api_key
    if api_key:
        alchemyObj = AlchemyAPI()
        alchemyObj.setAPIKey(api_key)
        try:
            result = alchemyObj.URLGetRankedConcepts(url)
            results += _list_alchemy_results(result)
            #result = alchemyObj.URLGetRankedKeywords(url)
            #results += _list_alchemy_results(result)
            #results = list(set(results))
            return results
        except:
            return results
    else:
        return results

def get_yql_subjects_remote(url):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.yahoo_api_key
    if api_key:
        y = yql.Public(api_key)
        query = '''select * from search.termextract where context in (
                select content from html where url="%s"
                )''' % url
        try:
            result = y.execute(query)
        except:
            return []
        return result.rows
    else:
        return []


def get_yql_subjects(text):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.yahoo_api_key
    if api_key:
        y = yql.Public(api_key)
        query = '''select * from search.termextract where context ="%s"
                ''' % text.replace('"',"'")
        try:
            result = y.execute(query)
        except:
            return []
        return result.rows
    else:
        return []

def get_ttn_subjects(text):
    ttn = TagTheNet()
    return ttn.analyze(text)

def get_ttn_subjects_remote(url):
    ttn = TagTheNet()
    return ttn.analyze_url(url)


def get_silcc_subjects(text):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.silcc_api_key
    if api_key == None:
        api_key=''
    url  = settings.silcc_url
    silcc = Silcc(api_key, url)
    try:
        return silcc.execute(text)
    except:
        return []

def get_calais_subjects(text, uid):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(ITagHelperSettingsSchema)
    api_key = settings.calais_api_key
    relevance = settings.calais_relevance
    subjects=[]
    if api_key:
        calais = Calais(api_key)
        try:
            result = calais.analyze(text, external_id = uid)
        except:
            return []
        #if hasattr( result, 'entities'):
        #    for entity in result.entities:
        #        if entity['_type'] in PREFERRED_ENTITIES:
        #            subjects.append(entity['name'])
        if hasattr( result, 'socialTag'):
            for tag in result.socialTag:
                if float(tag['importance']) > relevance:
                    subjects.append(tag['name'])
        #if hasattr( result, 'relations'):
        #    for fact in result.relations:
        #        if fact['_type'] in PREFERRED_FACTS:
        #            ft = fact.get(fact['_type'].lower())
        #            if ft:
        #                subjects.append(ft)
    return subjects



#call TagThe.net webservices
import urllib, urllib2
try:
    import simplejson as json
except ImportError:
    import json

class TagTheNet(object):
    url = 'http://tagthe.net/api/'

    def __init__(self):
        pass

    def _analyze(self, params):
        self.language = []
        self.topics = []
        self.location = []
        results = json.load(urllib2.urlopen(self.url, data=params))
        result = results.get('memes',[])
        for meme in result:
            dim = meme.get('dimensions')
            if dim:
                self.language = self.language + dim.get('language', [])
                self.topics = self.topics + dim.get('topic', [])
                self.location = self.location + dim.get('location', [])
        return self.topics + self.location

    def analyze_url(self, url):
        '''
        http://tagthe.net/api/?url=http://www.knallgrau.at/en&view=json
        '''
        keywords = []
        try:
            params = urllib.urlencode({'url':url, 'view':'json'})
            keywords = self._analyze(params)
        except:
            pass
        return keywords

    def analyze(self, text):
        '''
        http://tagthe.net/api/?text=Hello%20World!
        '''
        keywords = []
        try:
            params = urllib.urlencode({'text':text, 'view':'json'})
            keywords = self._analyze(params)
        except:
            pass
        return keywords




import urllib
import sys

from lxml import etree


class AlchemyAPI_Params(object):
  _url = ""
  _html = ""
  _text = ""
  _outputMode = ""
  _customParameters = ""
  def getUrl(self):
    return self._url
  def setUrl(self, url):
    self._url = url
  def getHtml(self):
    return self._html
  def setHtml(self, html):
    self._html = html
  def getText(self):
    return self._text
  def setText(self, text):
    self._text = text
  def getOutputMode(self):
    return self._mode
  def setOutputMode(self, mode):
    if mode != "xml":
      raise Exception, 'Error setting output mode.'
    self._outputMode = mode
  def getCustomParameters(self):
    return self._customParameters
  def setCustomParameters(self, *values):
    self._customParameters = ""
    for i in len(values):
      self._customParameters += "&" + values[i] + "=" + urllib.quote(values[i + 1])
      i = i + 1
  def getParameterString(self):
    retString = ""
    if self._url != "":
      retString += "&url=" + urllib.quote(self._url)
    if self._html != "":
      retString += "&html=" + urllib.quote(self._html)
    if self._text != "":
      retString += "&text=" + urllib.quote(self._text)
    if self._outputMode != "":
      retString += "&outputMode=" + urllib.quote(self._outputMode)
    if self._customParameters != "":
      retString += self._customParameters
    return retString


class AlchemyAPI_NamedEntityParams(AlchemyAPI_Params):
  _disambiguate = ""
  _linkedData = ""
  _coreference = ""
  _quotations = ""
  _sourceText = ""
  _showSourceText = ""
  _maxRetrieve = ""
  _baseUrl = ""
  _cQuery = ""
  _xPath = ""
  _sentiment = ""
  def getDisambiguate(self):
    return self._disambiguate
  def setDisambiguate(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Disambiguate.'
    self._disambiguate = setting
  def getLinkedData(self):
    return self._linkedData
  def setLinkedData(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting LinkedData.'
    self._linkedData = setting
  def getCoreference(self):
    return self._coreference
  def setCoreference(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Coreference.'
    self._coreference = setting
  def getQuotations(self):
    return self._quotations
  def setQuotations(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Quotations.'
    self._quotations = setting
  def getShowSourceText(self):
    return self._showSourceText
  def setShowSourceText(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting ShowSourceText.'
    self._showSourceText = setting
  def getSourceText(self):
    return self._quotations
  def setSourceText(self, setting):
    if setting != 'cleaned_or_raw':
      if setting != 'cleaned':
        if setting != 'raw':
          if setting != 'cquery':
            if setting != 'xpath':
              raise Exception, 'Error setting SourceText.'
    self._sourceText = setting
  def getMaxRetrieve(self):
    return self._maxRetrieve
  def setMaxRetrieve(self, setting):
    self._maxRetrieve = setting
  def getBaseUrl(self):
    return self._baseUrl
  def setBaseUrl(self, setting):
    self._baseUrl = setting
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getXPath(self):
    return self._xPath
  def setXPath(self, setting):
    self._xPath = setting
  def getSentiment(self):
    return self._sentiment
  def setSentiment(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Sentiment.'
    self._sentiment = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_NamedEntityParams, self).getParameterString()
    if self._disambiguate != "":
      retString += "&disambiguate=" + str(self._disambiguate)
    if self._linkedData != "":
      retString += "&linkedData=" + str(self._linkedData)
    if self._coreference != "":
      retString += "&coreference=" + str(self._coreference)
    if self._quotations != "":
      retString += "&quotations=" + str(self._quotations)
    if self._sourceText != "":
      retString += "&sourceText=" + urllib.quote(self._sourceText)
    if self._showSourceText != "":
      retString += "&showSourceText=" + str(self._showSourceText)
    if self._maxRetrieve != "":
      retString += "&maxRetrieve=" + str(self._maxRetrieve)
    if self._baseUrl != "":
      retString += "&baseUrl=" + urllib.quote(self._baseUrl)
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    if self._xPath != "":
      retString += "&xpath=" + urllib.quote(self._xPath)
    if self._sentiment != "":
      retString += "&sentiment=" + str(self._sentiment)
    return retString


class AlchemyAPI_CategoryParams(AlchemyAPI_Params):
  _sourceText = ""
  _baseUrl = ""
  _cQuery = ""
  _xPath = ""
  def getSourceText(self):
    return self._quotations
  def setSourceText(self, setting):
    if setting != 'cleaned_or_raw':
      if setting != 'cquery':
        if setting != 'xpath':
          raise Exception, 'Error setting SourceText.'
    self._sourceText = setting
  def getBaseUrl(self):
    return self._baseUrl
  def setBaseUrl(self, setting):
    self._baseUrl = setting
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getXPath(self):
    return self._xPath
  def setXPath(self, setting):
    self._xPath = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_CategoryParams, self).getParameterString()
    if self._sourceText != "":
      retString += "&sourceText=" + urllib.quote(self._sourceText)
    if self._baseUrl != "":
      retString += "&baseUrl=" + urllib.quote(self._baseUrl)
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    if self._xPath != "":
      retString += "&xpath=" + urllib.quote(self._xPath)
    return retString


class AlchemyAPI_LanguageParams(AlchemyAPI_Params):
  _sourceText = ""
  _cQuery = ""
  _xPath = ""
  def getSourceText(self):
    return self._sourceText
  def setSourceText(self, setting):
    if setting != 'cleaned_or_raw':
      if setting != 'cleaned':
        if setting != 'raw':
          if setting != 'cquery':
            if setting != 'xpath':
              raise Exception, 'Error setting SourceText.'
    self._sourceText = setting
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getXPath(self):
    return self._xPath
  def setXPath(self, setting):
    self._xPath = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_LanguageParams, self).getParameterString()
    if self._sourceText != "":
      retString += "&sourceText=" + urllib.quote(self._sourceText)
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    if self._xPath != "":
      retString += "&xpath=" + urllib.quote(self._xPath)
    return retString


class AlchemyAPI_ConceptParams(AlchemyAPI_Params):
  _sourceText = ""
  _showSourceText = ""
  _maxRetrieve = ""
  _cQuery = ""
  _xPath = ""
  _linkedData = ""
  def getSourceText(self):
    return self._sourceText
  def setSourceText(self, setting):
    if setting != 'cleaned_or_raw':
      if setting != 'cquery':
        if setting != 'xpath':
          raise Exception, 'Error setting SourceText.'
    self._sourceText = setting
  def getShowSourceText(self):
    return self._showSourceText
  def setShowSourceText(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting ShowSourceText.'
    self._showSourceText = setting
  def getMaxRetrieve(self):
    return self._maxRetrieve
  def setMaxRetrieve(self, setting):
    self._maxRetrieve = setting
  def getLinkedData(self):
    return self._linkedData
  def setLinkedData(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting LinkedData.'
    self._linkedData = setting
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getXPath(self):
    return self._xPath
  def setXPath(self, setting):
    self._xPath = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_ConceptParams, self).getParameterString()
    if self._sourceText != "":
      retString += "&sourceText=" + urllib.quote(self._sourceText)
    if self._showSourceText != "":
      retString += "&showSourceText=" + str(self._showSourceText)
    if self._maxRetrieve != "":
      retString += "&maxRetrieve=" + str(self._maxRetrieve)
    if self._linkedData != "":
      retString += "&linkedData=" + str(self._linkedData)
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    if self._xPath != "":
      retString += "&xpath=" + urllib.quote(self._xPath)
    return retString


class AlchemyAPI_KeywordParams(AlchemyAPI_Params):
  _sourceText = ""
  _showSourceText = ""
  _sentiment = ""
  _maxRetrieve = ""
  _baseUrl = ""
  _cQuery = ""
  _xPath = ""
  _keywordExtractMode = ""
  def getSourceText(self):
    return self._sourceText
  def setSourceText(self, setting):
    if setting != 'cleaned_or_raw':
      if setting != 'cquery':
        if setting != 'xpath':
          raise Exception, 'Error setting SourceText.'
    self._sourceText = setting
  def getShowSourceText(self):
    return self._showSourceText
  def setShowSourceText(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting ShowSourceText.'
    self._showSourceText = setting
  def getSentiment(self):
    return self._sentiment
  def setSentiment(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Sentiment.'
    self._sentiment = setting
  def getMaxRetrieve(self):
    return self._maxRetrieve
  def setMaxRetrieve(self, setting):
    self._maxRetrieve = setting
  def getBaseUrl(self):
    return self._baseUrl
  def setBaseUrl(self, setting):
    self._baseUrl = setting
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getXPath(self):
    return self._xPath
  def setXPath(self, setting):
    self._xPath = setting
  def getKeywordExtractMode(self):
    return self._keywordExtractMode
  def setKeywordExtractMode(self, setting):
    if setting != 'strict':
      if setting != 'normal':
        if setting != '':
          raise Exception, 'Error setting KeywordExtractMode.'
    self._keywordExtractMode = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_KeywordParams, self).getParameterString()
    if self._sourceText != "":
      retString += "&sourceText=" + urllib.quote(self._sourceText)
    if self._showSourceText != "":
      retString += "&showSourceText=" + str(self._showSourceText)
    if self._maxRetrieve != "":
      retString += "&maxRetrieve=" + str(self._maxRetrieve)
    if self._baseUrl != "":
      retString += "&baseUrl=" + urllib.quote(self._baseUrl)
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    if self._xPath != "":
      retString += "&xpath=" + urllib.quote(self._xPath)
    if self._keywordExtractMode != "":
      retString += "&keywordExtractMode=" + urllib.quote(self._keywordExtractMode)
    if self._sentiment != "":
      retString += "&sentiment=" + str(self._sentiment)
    return retString
	
	
class AlchemyAPI_RelationParams(AlchemyAPI_Params):
  _disambiguate = ""
  _linkedData = ""
  _coreference = ""
  _entities = ""
  _sentimentExcludeEntities = ""
  _requireEntities = ""
  _sourceText = ""
  _showSourceText = ""
  _maxRetrieve = ""
  _baseUrl = ""
  _cQuery = ""
  _xPath = ""
  _sentiment = ""
  def getDisambiguate(self):
    return self._disambiguate
  def setDisambiguate(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Disambiguate.'
    self._disambiguate = setting
  def getLinkedData(self):
    return self._linkedData
  def setLinkedData(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting LinkedData.'
    self._linkedData = setting
  def getCoreference(self):
    return self._coreference
  def setCoreference(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Coreference.'
    self._coreference = setting
  def getEntities(self):
    return self._entities
  def setEntities(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Entities.'
    self._entities = setting
  def getSentimentExcludeEntities(self):
    return self._sentimentExcludeEntities
  def setSentimentExcludeEntities(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting SentimentExcludeEntities.'
    self._sentimentExcludeEntities = setting
  def getRequireEntities(self):
    return self._requireEntities
  def setRequireEntities(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting RequireEntities.'
    self._requireEntities = setting
  def getShowSourceText(self):
    return self._showSourceText
  def setShowSourceText(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting ShowSourceText.'
    self._showSourceText = setting
  def getSourceText(self):
    return self._quotations
  def setSourceText(self, setting):
    if setting != 'cleaned_or_raw':
      if setting != 'cleaned':
        if setting != 'raw':
          if setting != 'cquery':
            if setting != 'xpath':
              raise Exception, 'Error setting SourceText.'
    self._sourceText = setting
  def getMaxRetrieve(self):
    return self._maxRetrieve
  def setMaxRetrieve(self, setting):
    self._maxRetrieve = setting
  def getBaseUrl(self):
    return self._baseUrl
  def setBaseUrl(self, setting):
    self._baseUrl = setting
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getXPath(self):
    return self._xPath
  def setXPath(self, setting):
    self._xPath = setting
  def getSentiment(self):
    return self._sentiment
  def setSentiment(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting Sentiment.'
    self._sentiment = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_RelationParams, self).getParameterString()
    if self._disambiguate != "":
      retString += "&disambiguate=" + str(self._disambiguate)
    if self._linkedData != "":
      retString += "&linkedData=" + str(self._linkedData)
    if self._coreference != "":
      retString += "&coreference=" + str(self._coreference)
    if self._entities != "":
      retString += "&entities=" + str(self._entities)
    if self._sentimentExcludeEntities != "":
      retString += "&sentimentExcludeEntities=" + str(self._sentimentExcludeEntities)
    if self._requireEntities != "":
      retString += "&requireEntities=" + str(self._requireEntities)
    if self._sourceText != "":
      retString += "&sourceText=" + urllib.quote(self._sourceText)
    if self._showSourceText != "":
      retString += "&showSourceText=" + str(self._showSourceText)
    if self._maxRetrieve != "":
      retString += "&maxRetrieve=" + str(self._maxRetrieve)
    if self._baseUrl != "":
      retString += "&baseUrl=" + urllib.quote(self._baseUrl)
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    if self._xPath != "":
      retString += "&xpath=" + urllib.quote(self._xPath)
    if self._sentiment != "":
      retString += "&sentiment=" + str(self._sentiment)
    return retString


class AlchemyAPI_TextParams(AlchemyAPI_Params):
  _useMetaData = ""
  _extractLinks = ""
  def getUseMetaData(self):
    return self._useMetaData
  def setUseMetaData(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting UseMetaData.'
    self._useMetaData = setting
  def getExtractLinks(self):
    return self._extractLinks
  def setExtractLinks(self, setting):
    if setting != 1:
      if setting != 0:
        raise Exception, 'Error setting ExtractLinks.'
    self._extractLinks = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_TextParams, self).getParameterString()
    if self._useMetaData != "":
      retString += "&useMetaData=" + str(self._useMetaData)
    if self._extractLinks != "":
      retString += "&extractLinks=" + str(self._extractLinks)
    return retString


class AlchemyAPI_ConstraintQueryParams(AlchemyAPI_Params):
  _cQuery = ""
  def getConstraintQuery(self):
    return self._cQuery
  def setConstraintQuery(self, setting):
    self._cQuery = setting
  def getParameterString(self):
    retString = super(AlchemyAPI_ConstraintQueryParams, self).getParameterString()
    if self._cQuery != "":
      retString += "&cquery=" + urllib.quote(self._cQuery)
    return retString


class AlchemyAPI:
  _apiKey = ""
  _hostPrefix = "access"
  def setAPIHost(self, apiHost):
    self._hostPrefix = apiHost;
    if len(self._hostPrefix) < 2:
      raise Exception, 'Error setting API host.'
  def setAPIKey(self, apiKey):
    self._apiKey = apiKey;
    if len(self._apiKey) < 5:
      raise Exception, 'Error setting API key.'
  def loadAPIKey(self, filename):
    file = open(filename, 'r')
    line = file.readline()
    self._apiKey = line.strip();
    if len(self._apiKey) < 5:
      raise Exception, 'Error loading API key.'
  def URLGetRankedNamedEntities(self, url, namedEntityParams=None):
    self.CheckURL(url)
    if namedEntityParams == None:
      namedEntityParams = AlchemyAPI_NamedEntityParams()
    namedEntityParams.setUrl(url)
    return self.GetRequest("URLGetRankedNamedEntities", "url", namedEntityParams)
  def HTMLGetRankedNamedEntities(self, html, url, namedEntityParams=None):
    self.CheckHTML(html, url)
    if namedEntityParams == None:
      namedEntityParams = AlchemyAPI_NamedEntityParams()
    namedEntityParams.setUrl(url)
    namedEntityParams.setHtml(html)
    return self.PostRequest("HTMLGetRankedNamedEntities", "html", namedEntityParams)
  def TextGetRankedNamedEntities(self, text, namedEntityParams=None):
    self.CheckText(text)
    if namedEntityParams == None:
      namedEntityParams = AlchemyAPI_NamedEntityParams()
    namedEntityParams.setText(text)
    return self.PostRequest("TextGetRankedNamedEntities", "text", namedEntityParams)
  def URLGetRankedConcepts(self, url, conceptParams=None):
    self.CheckURL(url)
    if conceptParams == None:
      conceptParams = AlchemyAPI_ConceptParams()
    conceptParams.setUrl(url)
    return self.GetRequest("URLGetRankedConcepts", "url", conceptParams)
  def HTMLGetRankedConcepts(self, html, url, conceptParams=None):
    self.CheckHTML(html, url)
    if conceptParams == None:
      conceptParams = AlchemyAPI_ConceptParams()
    conceptParams.setUrl(url)
    conceptParams.setHtml(html)
    return self.PostRequest("HTMLGetRankedConcepts", "html", conceptParams)
  def TextGetRankedConcepts(self, text, conceptParams=None):
    self.CheckText(text)
    if conceptParams == None:
      conceptParams = AlchemyAPI_ConceptParams()
    conceptParams.setText(text)
    return self.PostRequest("TextGetRankedConcepts", "text", conceptParams)
  def URLGetRankedKeywords(self, url, keywordParams=None):
    self.CheckURL(url)
    if keywordParams == None:
      keywordParams = AlchemyAPI_KeywordParams()
    keywordParams.setUrl(url)
    return self.GetRequest("URLGetRankedKeywords", "url", keywordParams)
  def HTMLGetRankedKeywords(self, html, url, keywordParams=None):
    self.CheckHTML(html, url)
    if keywordParams == None:
      keywordParams = AlchemyAPI_KeywordParams()
    keywordParams.setUrl(url)
    keywordParams.setHtml(html)
    return self.PostRequest("HTMLGetRankedKeywords", "html", keywordParams)
  def TextGetRankedKeywords(self, text, keywordParams=None):
    self.CheckText(text)
    if keywordParams == None:
      keywordParams = AlchemyAPI_KeywordParams()
    keywordParams.setText(text)
    return self.PostRequest("TextGetRankedKeywords", "text", keywordParams)
  def URLGetLanguage(self, url, languageParams=None):
    self.CheckURL(url)
    if languageParams == None:
      languageParams = AlchemyAPI_LanguageParams()
    languageParams.setUrl(url)
    return self.GetRequest("URLGetLanguage", "url", languageParams)
  def HTMLGetLanguage(self, html, url, languageParams=None):
    self.CheckHTML(html, url)
    if languageParams == None:
      languageParams = AlchemyAPI_LanguageParams()
    languageParams.setUrl(url)
    languageParams.setHtml(html)
    return self.PostRequest("HTMLGetLanguage", "html", languageParams)
  def TextGetLanguage(self, text, languageParams=None):
    self.CheckText(text)
    if languageParams == None:
      languageParams = AlchemyAPI_LanguageParams()
    languageParams.setText(text)
    return self.PostRequest("TextGetLanguage", "text", languageParams)
  def URLGetCategory(self, url, categParams=None):
    self.CheckURL(url)
    if categParams == None:
      categParams = AlchemyAPI_CategoryParams()
    categParams.setUrl(url)
    return self.GetRequest("URLGetCategory", "url", categParams)
  def HTMLGetCategory(self, html, url, categParams=None):
    self.CheckHTML(html, url)
    if categParams == None:
      categParams = AlchemyAPI_CategoryParams()
    categParams.setUrl(url)
    categParams.setHtml(html)
    return self.PostRequest("HTMLGetCategory", "html", categParams)
  def TextGetCategory(self, text, categParams=None):
    self.CheckText(text)
    if categParams == None:
      categParams = AlchemyAPI_CategoryParams()
    categParams.setText(text)
    return self.PostRequest("TextGetCategory", "text", categParams)
  def URLGetText(self, url, textParams=None):
    self.CheckURL(url)
    if textParams == None:
      textParams = AlchemyAPI_TextParams()
    textParams.setUrl(url)
    return self.GetRequest("URLGetText", "url", textParams)
  def HTMLGetText(self, html, url, textParams=None):
    self.CheckHTML(html, url)
    if textParams == None:
      textParams = AlchemyAPI_TextParams()
    textParams.setUrl(url)
    textParams.setHtml(html)
    return self.PostRequest("HTMLGetText", "html", textParams)
  def URLGetRawText(self, url, baseParams=None):
    self.CheckURL(url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    return self.GetRequest("URLGetRawText", "url", baseParams)
  def HTMLGetRawText(self, html, url, baseParams=None):
    self.CheckHTML(html, url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    baseParams.setHtml(html)
    return self.PostRequest("HTMLGetRawText", "html", baseParams)
  def URLGetTitle(self, url, baseParams=None):
    self.CheckURL(url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    return self.GetRequest("URLGetTitle", "url", baseParams)
  def HTMLGetTitle(self, html, url, baseParams=None):
    self.CheckHTML(html, url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    baseParams.setHtml(html)
    return self.PostRequest("HTMLGetTitle", "html", baseParams)
  def URLGetFeedLinks(self, url, baseParams=None):
    self.CheckURL(url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    return self.GetRequest("URLGetFeedLinks", "url", baseParams)
  def HTMLGetFeedLinks(self, html, url, baseParams=None):
    self.CheckHTML(html, url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    baseParams.setHtml(html)
    return self.PostRequest("HTMLGetFeedLinks", "html", baseParams)
  def URLGetMicroformats(self, url, baseParams=None):
    self.CheckURL(url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    return self.GetRequest("URLGetMicroformatData", "url", baseParams)
  def HTMLGetMicroformats(self, html, url, baseParams=None):
    self.CheckHTML(html, url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    baseParams.setHtml(html)
    return self.PostRequest("HTMLGetMicroformatData", "html", baseParams)
  def URLGetConstraintQuery(self, url, query, cQueryParams=None):
    self.CheckURL(url)
    if len(query) < 2:
      raise Exception, 'Invalid constraint query specified.'
    if cQueryParams == None:
      cQueryParams = AlchemyAPI_ConstraintQueryParams()
    cQueryParams.setUrl(url)
    cQueryParams.setConstraintQuery(query)
    return self.GetRequest("URLGetConstraintQuery", "url", cQueryParams)
  def HTMLGetConstraintQuery(self, html, url, query, cQueryParams=None):
    self.CheckHTML(html, url)
    if len(query) < 2:
      raise Exception, 'Invalid constraint query specified.'
    if cQueryParams == None:
      cQueryParams = AlchemyAPI_ConstraintQueryParams()
    cQueryParams.setUrl(url)
    cQueryParams.setHtml(html)
    cQueryParams.setConstraintQuery(query)
    return self.PostRequest("HTMLGetConstraintQuery", "html", cQueryParams)
  def URLGetTextSentiment(self, url, baseParams=None):
    self.CheckURL(url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    return self.GetRequest("URLGetTextSentiment", "url", baseParams)
  def HTMLGetTextSentiment(self, html, url, baseParams=None):
    self.CheckHTML(html, url)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setUrl(url)
    baseParams.setHtml(html)
    return self.PostRequest("HTMLGetTextSentiment", "html", baseParams)
  def TextGetTextSentiment(self, text, baseParams=None):
    self.CheckText(text)
    if baseParams == None:
      baseParams = AlchemyAPI_Params()
    baseParams.setText(text)
    return self.PostRequest("TextGetTextSentiment", "text", baseParams)
  def URLGetRelations(self, url, relationParams=None):
    self.CheckURL(url)
    if relationParams == None:
      relationParams = AlchemyAPI_RelationParams()
    relationParams.setUrl(url)
    return self.GetRequest("URLGetRelations", "url", relationParams)
  def HTMLGetRelations(self, html, url, relationParams=None):
    self.CheckHTML(html, url)
    if relationParams == None:
      relationParams = AlchemyAPI_RelationParams()
    relationParams.setUrl(url)
    relationParams.setHtml(html)
    return self.PostRequest("HTMLGetRelations", "html", relationParams)
  def TextGetRelations(self, text, relationParams=None):
    self.CheckText(text)
    if relationParams == None:
      relationParams = AlchemyAPI_RelationParams()
    relationParams.setText(text)
    return self.PostRequest("TextGetRelations", "text", relationParams)
  def CheckText(self, text):
    if len(self._apiKey) < 5:
      raise Exception, 'Please load an API key.'
    if len(text) < 5:
      raise Exception, 'Please specify some text to analyze.'
  def CheckHTML(self, html, url):
    if len(self._apiKey) < 5:
      raise Exception, 'Please load an API key.'
    if len(html) < 10:
      raise Exception, 'Please specify a HTML document to analyze.'
    if len(url) < 10:
      raise Exception, 'Please specify a URL to analyze.'
  def CheckURL(self, url):
    if len(self._apiKey) < 5:
      raise Exception, 'Please load an API key.'
    if len(url) < 10:
      raise Exception, 'Please specify a URL to analyze.'
  def PostRequest(self, apiCall, apiPrefix, paramObject):
    endpoint = 'http://' + self._hostPrefix + '.alchemyapi.com/calls/' + apiPrefix + '/' + apiCall
    argText = 'apikey=' + self._apiKey + paramObject.getParameterString()
    handle = urllib.urlopen(endpoint, argText)
    result = handle.read()
    handle.close()
    xpathQuery = '/results/status'
    nodes = etree.fromstring(result).xpath(xpathQuery)
    if nodes[0].text != "OK":
      raise Exception, 'Error making API call.'
    return result
  def GetRequest(self, apiCall, apiPrefix, paramObject):
    endpoint = 'http://' + self._hostPrefix + '.alchemyapi.com/calls/' + apiPrefix + '/' + apiCall
    endpoint += '?apikey=' + self._apiKey + paramObject.getParameterString()
    handle = urllib.urlopen(endpoint)
    result = handle.read()
    handle.close()
    xpathQuery = '/results/status'
    nodes = etree.fromstring(result).xpath(xpathQuery)
    if nodes[0].text != "OK":
      raise Exception, 'Error making API call.'
    return result



# -*- coding: utf-8 -*-
"""
This module contains the tool of collective.taghelper
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.3'

long_description = (
read('README.txt')
        + '\n' +
        'Change history\n'
        '**************\n'
        + '\n' +
        read('CHANGES.txt')
        #+ '\n' +
        #'Detailed Documentation\n'
        #'**********************\n'
        #+ '\n' +
        #read('collective', 'taghelper', 'README.txt')
        #+ '\n' +
        #'Contributors\n'
        #'************\n'
        #+ '\n' +
        #read('CONTRIBUTORS.txt')
        + '\n' +
        'Download\n'
        '********\n')

tests_require = ['zope.testing']

setup(name='collective.taghelper',
      version=version,
      description="""The act of tagging content is tedious and humans will often fail to do it. Taghelper examines the content and extracts the keywords that are most relevant.""",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.0',
        'Framework :: Plone :: 4.1',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Development Status :: 5 - Production/Stable'
        ],
      keywords='',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='http://plone.org/products/collective.taghelper/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        # -*- Extra requirements: -*-
                        'yql', 'plone.app.registry',
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite='collective.taghelper.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      #setup_requires=["PasteScript"],
      #paster_plugins=["ZopeSkel"],
      )
